mod Maux logic: 'cl'

incl Mtex

incl Mtexbt

rem 
  \para \bf  Numeric programs.  \end 

rem 
  \para \it  Maximum.  \end 

fun/2 Max 'Tex_f2_n_max'
  Max(x,y) = x <- x >= y
  Max(x,y) = y <- x < y

rem 
  \para \it  Exponentiation of two.  \end 

fun Exp2 'Tex_f2_n_exp2'
  Exp2(0) = 1
  Exp2(x+1) = 2*Exp2(x)

thm Exp2_is_not_zero
  Exp2(x) != 0
proof 
 ind N; x @ 0; x
  proved
  proved..

thm Exp2_is_one
  Exp2(x) = 1 <-> x = 0
proof 
 case N; x @ 0; x1
  proved
  proved..

thm Exp2_gt_one
  Exp2(x) > 1 <-> x != 0
proof 
 cut* Exp2(x) > 1 <-> Exp2(x) != 0 & Exp2(x) != 1
  weak* Exp2(x) > 1 <-> x != 0
   split* Exp2(x) > 1 <-> Exp2(x) != 0 & Exp2(x) != 1.2,0,0
    proved
    proved..
  use Exp2_is_not_zero; x
   use Exp2_is_one.0; x proved....

thm Exp2_plus
  Exp2(x+y) = Exp2(x)*Exp2(y)
proof 
 ind N; x @ 0; x
  proved
  proved..

thm Exp2_minus
  x >= y -> Exp2(x-y) = Exp2(x)/Exp2(y)
proof 
 ind N; x; y @ 0; x
  proved
  case N; y @ 0; y1
   proved
   inst \a y(x >= y -> Exp2(x-y) = Exp2(x)/Exp2(y)); y1 proved....

thm Exp2_cancel_eq
  Exp2(x) = Exp2(y) <-> x = y
proof 
 ind N; x; y @ 0; x
  use Exp2_is_one.0; y proved.
  case N; y @ 0; y1
   proved
   inst \a y(Exp2(x) = Exp2(y) <-> x = y).0; y1 proved....

thm Exp2_cancel_le
  Exp2(x) <= Exp2(y) <-> x <= y
proof 
 ind N; x; y @ 0; x
  use Exp2_is_not_zero; y proved.
  case N; y @ 0; y1
   use Exp2_is_not_zero; x proved.
   inst \a y(Exp2(x) <= Exp2(y) <-> x <= y).0; y1 proved....

thm Exp2_cancel_lt
  Exp2(x) < Exp2(y) <-> x < y
proof 
 ind N; x; y @ 0; x
  use Exp2_gt_one.0; y proved.
  case N; y @ 0; y1
   use Exp2_is_not_zero; x proved.
   inst \a y(Exp2(x) < Exp2(y) <-> x < y).0; y1 proved....

rem 
  \para \bf  Lists.  \end 

rem 
  \para \it  Interval.  \end 

appldisp/2 Tex_f2_ln_interval_co
  Fenced(Op('[',0),Infix(Arg(0),25,3,Id(0,'  .. ',0),Arg(1)),Op(')',0))

fun/2 Interval 'Tex_f2_ln_interval_co'
  Interval(m,n) = 0 <- m >= n
  Interval(m,n) = m,Interval(m+1,n) <- m < n

thm Interval_conc
  k <= m & m <= n -> Interval(k,m)++Interval(m,n) = Interval(k,n)
proof 
 indm m-k @ m1,k1
  case Dich; m,k @ x,y; x,y
   proved
   inst \a m1\a k1(k+(m1-k1) < m -> k1 <= m1 & m1 <= n -> 
                   Interval(k1,m1)++Interval(m1,n) = Interval(k1,n)); m; k+1 proved....

rem 
  \para \bf  Binary trees.  \end 

rem 
  \para \it  Arithmetization.  \end 

fun/0 E 'Tex_f0_bt_e'
  E = 0,0

fun/3 Nd 'Tex_f3_bt_nd'
  Nd(x,l,r) = 1,x,l,r

pred Bt 
  Bt(E)
  Bt Nd(x,l,r) <- N(x) & Bt(l) & Bt(r)

rem 
  \para \it  Size and depth of binary trees.  \end 

fun Sz 'Tex_f1_bt_sz'
  Sz(E) = 0
  Sz Nd(x,l,r) = Sz(l)+Sz(r)+1

fun Dp 'Tex_f1_bt_dp'
  Dp(E) = 0
  Dp Nd(x,l,r) = Max(Dp(l),Dp(r))+1

thm Sz_is_zero
  Bt(t) -> Sz(t) = 0 <-> t = E
proof 
 case Bt; t @ 0; x,l,r
  proved
  proved..

thm Dp_is_zero
  Bt(t) -> Dp(t) = 0 <-> t = E
proof 
 case Bt; t @ 0; x,l,r
  proved
  proved..

thm Dp_sz
  Bt(t) -> Dp(t) <= Sz(t) & Sz(t) < Exp2 Dp(t)
proof 
 ind Bt; t @ 0; x,l,r
  proved
  case Dich; Dp(r),Dp(l) @ x1,y; x1,y
   use Exp2_cancel_le.2,1,0; Dp(r); Dp(l) proved.
   use Exp2_cancel_le.2,1,0; Dp(l); Dp(r) proved....

rem 
  \para \it  Isomorphic binary trees.  \end 

pred/2 Isomorphic 'Tex_p2_bt_isomorphic'
  Isomorphic(E,E)
  Isomorphic(Nd(x1,l1,r1),Nd(x2,l2,r2)) <- Isomorphic(l1,l2) & Isomorphic(r1,r2)

thm Isomorphic_refl
  Bt(t) -> Isomorphic(t,t)
proof 
 ind Bt; t @ 0; x,l,r
  proved
  proved..

thm Isomorphic_sym
  Isomorphic(t1,t2) -> Isomorphic(t2,t1)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

thm Isomorphic_trans
  Isomorphic(t1,t2) & Isomorphic(t2,t3) -> Isomorphic(t1,t3)
proof 
 ind Isomorphic; t1,t2; t3 @ 0; x1,l1,r1,x2,l2,r2
  proved
  case Isomorphic; Nd(x2,l2,r2),t3 @ 0; x22,l22,r22,x3,l3,r3
   proved
   inst \a t3(Isomorphic(l22,t3) -> Isomorphic(l1,t3)); l3
    inst \a t3(Isomorphic(r22,t3) -> Isomorphic(r1,t3)); r3 proved.....

thm Isomorphic_bt
  Bt(t1) & Isomorphic(t1,t2) -> Bt(t2)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

thm Isomorphic_sz
  Isomorphic(t1,t2) -> Sz(t1) = Sz(t2)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

thm Isomorphic_dp
  Isomorphic(t1,t2) -> Dp(t1) = Dp(t2)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

rem 
  \para \it  Traversals of binary trees.  \end 

fun Preorder 
  Preorder(E) = 0
  Preorder Nd(x,l,r) = (x,0)++Preorder(l)++Preorder(r)

fun Inorder 
  Inorder(E) = 0
  Inorder Nd(x,l,r) = Inorder(l)++(x,0)++Inorder(r)

fun Postorder 
  Postorder(E) = 0
  Postorder Nd(x,l,r) = Postorder(l)++Postorder(r)++(x,0)

